@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2019-2023 Nicola Di Lieto <nicola.dilieto@gmail.com>
+ * Copyright (C) 2019-2024 Nicola Di Lieto <nicola.dilieto@gmail.com>
  *
  * This file is part of uacme.
  *
@@ -20,12 +20,14 @@
 
 #include "config.h"
 #include <err.h>
+#include <regex.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
 
 #include "curlwrap.h"
+#include "msg.h"
 
 curldata_t *curldata_calloc(void)
 {
@@ -89,6 +91,30 @@ static size_t curl_wcb(void *ptr, size_t size, size_t n, void *userdata)
     return size * n;
 }
 
+static void curl_env(CURL *curl)
+{
+    const char *cainfo = getenv("UACME_CAINFO");
+    const char *capath = getenv("UACME_CAPATH");
+    const char *dnssrv = getenv("UACME_DNS_SERVERS");
+    const char *iface = getenv("UACME_INTERFACE");
+    const char *proxy = getenv("UACME_PROXY");
+
+    curl_easy_setopt(curl, CURLOPT_USERAGENT,
+            "uacme/" VERSION " (https://github.com/ndilieto/uacme)");
+    if (g_loglevel > 3)
+        curl_easy_setopt(curl, CURLOPT_VERBOSE, 1L);
+    if (cainfo)
+        curl_easy_setopt(curl, CURLOPT_CAINFO, cainfo);
+    if (capath)
+        curl_easy_setopt(curl, CURLOPT_CAPATH, capath);
+    if (dnssrv)
+        curl_easy_setopt(curl, CURLOPT_DNS_SERVERS, dnssrv);
+    if (iface)
+        curl_easy_setopt(curl, CURLOPT_INTERFACE, iface);
+    if (proxy)
+        curl_easy_setopt(curl, CURLOPT_PROXY, proxy);
+}
+
 curldata_t *curl_get(const char *url)
 {
     curldata_t *c = NULL;
@@ -106,13 +132,12 @@ curldata_t *curl_get(const char *url)
             curl_easy_cleanup(curl);
             return NULL;
         }
+        curl_env(curl);
         curl_easy_setopt(curl, CURLOPT_URL, url);
         curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, curl_wcb);
         curl_easy_setopt(curl, CURLOPT_WRITEDATA, c);
         curl_easy_setopt(curl, CURLOPT_HEADERFUNCTION, curl_hcb);
         curl_easy_setopt(curl, CURLOPT_HEADERDATA, c);
-        curl_easy_setopt(curl, CURLOPT_USERAGENT,
-                "uacme/" VERSION " (https://github.com/ndilieto/uacme)");
         res = curl_easy_perform(curl);
         if (res != CURLE_OK) {
             warnx("curl_get: GET %s failed: %s", url,
@@ -155,13 +180,12 @@ curldata_t *curl_post(const char *url, void *post_data, size_t post_size,
             curl_easy_cleanup(curl);
             return NULL;
         }
+        curl_env(curl);
         curl_easy_setopt(curl, CURLOPT_URL, url);
         curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, curl_wcb);
         curl_easy_setopt(curl, CURLOPT_WRITEDATA, c);
         curl_easy_setopt(curl, CURLOPT_HEADERFUNCTION, curl_hcb);
         curl_easy_setopt(curl, CURLOPT_HEADERDATA, c);
-        curl_easy_setopt(curl, CURLOPT_USERAGENT,
-                "uacme/" VERSION " (https://github.com/ndilieto/uacme)");
         curl_easy_setopt(curl, CURLOPT_POSTFIELDS, post_data);
         curl_easy_setopt(curl, CURLOPT_POSTFIELDSIZE, post_size);
         va_start(ap, header);
@@ -193,3 +217,27 @@ curldata_t *curl_post(const char *url, void *post_data, size_t post_size,
     }
     return c;
 }
+
+char *find_header(const char *headers, const char *name)
+{
+    char *regex = NULL;
+    if (asprintf(&regex, "^%s:[ \t]*(.*)\r\n", name) < 0) {
+        warnx("find_header: asprintf failed");
+        return NULL;
+    }
+    char *ret = NULL;
+    regex_t reg;
+    if (regcomp(&reg, regex, REG_EXTENDED | REG_ICASE | REG_NEWLINE)) {
+        warnx("find_header: regcomp failed");
+    } else {
+        regmatch_t m[2];
+        if (regexec(&reg, headers, 2, m, 0) == 0) {
+            ret = strndup(headers + m[1].rm_so, m[1].rm_eo - m[1].rm_so);
+            if (!ret)
+                warn("find_header: strndup failed");
+        }
+    }
+    free(regex);
+    regfree(&reg);
+    return ret;
+}
